<?php

if (passive_authcache_builtin_serve_proxy()) {
  exit;
}

/**
 * Serves Authcache built-in backend as passive proxy.
 */
function passive_authcache_builtin_serve_proxy() {
  if (passive_is_passive_proxy()) {
    // Serve passive response.
    if (passive_check_passive_request()) {
      passive_authcache_builtin_serve_passive_response();
    }
    // Obtain the passive key for the current request.
    elseif ($passive_key = passive_authcache_get_passive_key()) {
      passive_serve_passive_prompt($passive_key);
      return TRUE;
    }
  }

  return FALSE;
}

/**
 * Checks backend response and alters Authcache built-in behavior.
 */
function passive_authcache_builtin_serve_passive_response() {
  // Force Authcache built-in backend to use Vary: X-Passive-Key.
  global $conf;
  $conf['authcache_builtin_vary'] = 'X-Passive-Key';
}

/**
 * Obtains the Authcache key if the current request warrants a passive prompt.
 */
function passive_authcache_get_passive_key() {
  global $base_root;

  if (!function_exists('authcache_backend_init')) {
    return FALSE;
  }

  // Attempt to construct Authcache key.
  $key = FALSE;
  if (isset($_COOKIE[session_name()])) {
    $session_id = $_COOKIE[session_name()];
    if ($cache = cache_get($base_root . ':' . $session_id, 'cache_authcache_key')) {
      $key = $cache->data;
    }
  }
  else {
    // Pass non-session requests through to regular page cache.
    return FALSE;
  }

  // Attempt to determine if page cache is available.
  if ($key !== FALSE) {
    $cid = $key . request_uri();
    $cache = cache_get($cid, 'cache_page');
    $buffer = variable_get('passive_prompt_cache_buffer', 2);
    $cache_lifetime = variable_get('cache_lifetime', 0);
    if ($cache) {
      $expire = max($cache->created + $cache_lifetime, $cache->expire);
      if ($expire > REQUEST_TIME + $buffer) {
        return $key;
      }
    }
  }

  // Proceed as normal by default.
  return FALSE;
}
